const requireOptional = require("tp-requireoptional");
const DB = requireOptional("tp-db");
const email = requireOptional("tp-email");

class Err extends Error {
    constructor(message = 'An undefined error occurred.', data, ...params) {
        // Pass remaining arguments (including vendor specific ones) to parent constructor
        super(...params);

        // Maintains proper stack trace for where our error was thrown (only available on V8)
        if (Error.captureStackTrace) {
            Error.captureStackTrace(this, Err);
        }

        // Custom debugging information
        this.message = message;
        this.data = data;
        this.date = new Date();
        this.stackTrace = this.stack.split("\n");

        if (DB) {
            this.saveErr(this);
        }

        if (email) {
            this.sendEmail(this);
        }
    }

    saveErr(error) {
        let errorModel = require("./model/errors");

        DB.req("mongo", "errors", "new", error, function(err) {
            if (err) {
                console.error("Unable to save error report in database.");
                console.error(err);
            }
        }, {
            model: errorModel
        });
    }

    sendEmail(error) {
        let body = "<b>The following error occurred:</b><br>" + error.message + "<br>&nbsp;<br><b>Error data:</b><br>" + JSON.stringify(error.data) + "<br>&nbsp;<br><b>Stack trace:</b><br>";
        for (let t=0; t< this.stackTrace.length; t++) {
            body += this.stackTrace[t] + "<br>";
        }

        email.send(
            {
                to: "kriss@trespass.co.uk",
                subject: "ERROR: " + error.message,
                body: body
            },
            "error",
            function (err) {
                if (err) {
                    console.error("Unable to send error email.");
                    console.error(err);
                }
            });
    }
}

module.exports = Err;
